import {
  Entity,
  Column,
  PrimaryGeneratedColumn,
  ManyToOne,
  OneToMany,
  Timestamp,
} from 'typeorm';
import { PersonEntity } from 'src/user/user.entity';

// table that stores all created games
@Entity('Game')
export class GameEntity {
  @PrimaryGeneratedColumn('uuid') id: string;
  @Column('text') name: string;
  @Column('text') desc: string;
  @Column('json') center: JSON;
  @Column('json') map: JSON;
  @Column('timestamp') startdate: Timestamp;
  @Column('timestamp') enddate: Timestamp;
  @Column("text", {array: true}) passwords: string[];
  @OneToMany(type => FactionEntity, faction => faction.game)
  factions: FactionEntity[];
  @OneToMany(type => Game_PersonEntity, game_persons => game_persons.game)
  game_persons: Game_PersonEntity[];


  gameObject() {
    const { id, name } = this;
    return { id, name };
  }
}

// table that stores all factions created for games
@Entity('Faction')
export class FactionEntity {
  @PrimaryGeneratedColumn('uuid') id: string;
  @Column('text') name: string;
  @ManyToOne(type => GameEntity, game => game.factions)
  game: GameEntity;
  @OneToMany(type => Game_PersonEntity, game_persons => game_persons.faction)
  game_persons: Game_PersonEntity[];

}

// table that stores players associated with particular game
@Entity('Game_Person')
export class Game_PersonEntity {
    @PrimaryGeneratedColumn('uuid') gameId: string;
    @ManyToOne(type => FactionEntity, faction => faction.game_persons)
    faction: FactionEntity;
    @ManyToOne(type => GameEntity, game => game.game_persons)
    game: GameEntity;
    @ManyToOne(type => PersonEntity, person => person.game_persons)
    person: PersonEntity;
    @ManyToOne(type => PersonRoleEntity, person_role => person_role.game_persons)
    person_role: PersonRoleEntity;
    @ManyToOne(type => CoordinateEntity, game_person_coordinates => game_person_coordinates.game_persons)
    game_person_coordinates: CoordinateEntity;
}

@Entity('ObjectivePoint')
export class ObjectivePointEntity {
    @PrimaryGeneratedColumn('uuid') objectivePointId: string;
    @Column({type: 'text'}) objectivePointDescription: string;
    @Column({type: 'float'}) objectivePointMultiplier: number;

    @ManyToOne(type => CoordinateEntity, coordinate => coordinate.objective_points)
    coordinate: CoordinateEntity;
    @ManyToOne(type => GameEntity, game => game.objective_points)
    game: GameEntity;
    @OneToMany(type => ObjectivePoint_HistoryEntity, op_history => op_history.objective_point)
    op_history: Game_PersonEntity[];
}

@Entity('ObjectivePoint_History')
export class ObjectivePoint_HistoryEntity {
    @PrimaryGeneratedColumn('uuid') oP_HistoryId: string;
    @Column({type: 'timestamp'}) oP_HistoryTimestamp: Timestamp;
    @Column({}) oP_HistoryStatus: number;

    @ManyToOne(type => ObjectivePointEntity, objective_point => objective_point.op_history)
    objective_point: ObjectivePointEntity;
}